#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime

# not use telegram_util once, because that one mess up with markdown
# also, this way reduce dependency
def cutText(text, cut):
	if len(text) <= cut + 3:
		return text
	return text[:cut] + '...'

def textJoin(*parts):
	result = ' '.join(parts)
	parts = [x for x in result.split() if x]
	return ' '.join(parts)

def getText(*soups):
	result = []
	for soup in soups:
		if soup:
			for br in soup.find_all("br"):
				br.replace_with("\n")
			result.append(soup.text)
	return textJoin(*result)


import cached_url
from bs4 import BeautifulSoup
from datetime import datetime

def getSoup(url, force_cache=True):
	return BeautifulSoup(cached_url.get(url, force_cache=force_cache),
		'html.parser')

def getField(soup, *fields):
	for field in fields:
		result = soup.find('div', class_=field)
		if result:
			return result

def getAField(soup, field):
	try:
		result = soup.find('a', class_=field)['href']
		return result.split('/')
	except:
		...

def getAFieldSuffix(soup, field):
	pieces = getAField(soup, field)
	try:
		return pieces[-1]
	except:
		...

def getForwardFrom(soup):
	pieces = getAField(soup, 'tgme_widget_message_forwarded_from_name')
	try:
		int(pieces[-1])
		return pieces[-2]
	except:
		...

def getTime(soup):
	try:
		return int(datetime.strptime(soup.find('a', 
			class_='tgme_widget_message_date').find('time')[
			'datetime'][:-6], '%Y-%m-%dT%H:%M:%S').timestamp())
	except:
		return 0

def getPostId(soup):
	post_link = soup.find('a', 
		class_='tgme_widget_message_date')['href']
	return int(post_link.split('/')[-1])

def getLinks(soup):
	if not soup:
		return []
	return [item['href'] for item in 
		soup.find_all('a') if item.get('href')]

def isGroup(soup):
	if soup.find('div', class_='tgme_page_context_action'):
		return False
	return 'online</div>' in str(soup)
	
	


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

name = 'webgram'
from .ssoup import getSoup, getPostId
from .model import Post, getPostFromSoup

def _yieldPosts(name, soup):
	yield getPostFromSoup(name, soup)
	for sub_soup in soup.find_all('div', class_='tgme_widget_message_bubble'):
		post = getPostFromSoup(name, sub_soup)
		try:
			# in rare cases, tgme_widget_message_date field does no exist
			post.post_id = getPostId(sub_soup)
			yield post
		except:
			...

def _getPostsSoup(name, post_id, direction, force_cache):
	link = 'https://t.me/s/' + name
	if post_id:
		link += '?%s=%d' % (direction, post_id)
	return getSoup(link, force_cache=force_cache or post_id)

def getPosts(name, post_id=None, direction='after', force_cache = False):
	soup = _getPostsSoup(name, post_id, direction, force_cache)
	return list(_yieldPosts(name, soup))

def getPost(name, post_id):
	soup = getSoup('https://t.me/%s/%d?embed=1' % (name, post_id))
	post = getPostFromSoup(name, soup)
	post.post_id = post_id
	return post

def get(name):
	soup = getSoup('https://t.me/' + name)
	post = getPostFromSoup(name, soup)
	if not post.title or 'Send Message' in str(
		soup.find('a', class_='tgme_action_button_new')):
		post.exist = False
	return post

def yieldReferers(post):
	for name in post.yieldRefers():
		if get(name).exist:
			yield name




from .ssoup import getField, getTime, getForwardFrom, getLinks, isGroup, getAFieldSuffix
from .util import getText, cutText, textJoin

def isValidName(candidate):
	if len(candidate) > 40:
		return False
	for c in '?#=!%,.':
		if c in candidate:
			return False
	return True

def isValidUsername(candidate):
	if not candidate:
		return False
	try:
		int(candidate)
		return False
	except:
		...
	return isValidName(candidate)

class Post(object): # can be a post or channel info wrap
	def __init__(self, channel):
		self.channel = channel
		self.post_id = 0
		self.exist = True

	def yieldRefers(self):
		if self.forward_from:
			yield self.forward_from
		soup = self.description if self.isChannel() else self.text
		for link in getLinks(soup):
			if 't.me' in link:
				parts = link.split('t.me')[-1].split('/')
				if len(parts) <= 1:
					continue
				candidate = parts[1]
				if isValidName(candidate):
					yield candidate

	def isChannel(self):
		return self.post_id == 0

	def getMaintext(self, cut = 20, channel_cut = 15):
		if self.isChannel():
			return cutText(getText(self.title), channel_cut)
		return cutText(self._getIndex(), cut)

	def _getIndex(self):
		if self.isChannel():
			return getText(self.title, self.description)
		if not self.text or not self.link:
			return getText(self.file, self.link, self.preview, 
				self.text, self.poll)
		textLink = None
		for item in self.text.find_all('a'):
			textLink = getText(item)
			if textLink:
				break
		if not textLink:
			return getText(self.file, self.link, self.preview, self.text)
		text = getText(self.text)
		first_part = text.split(textLink)[0]
		second_part = text[len(first_part):]
		return textJoin(getText(self.file), first_part, 
			getText(self.link, self.preview), second_part)

	def getIndex(self):
		raw = []
		if self.isChannel():
			if self.is_group:
				raw.append('isGroup')
		else:
			if len(getLinks(self.text)) > 0:
				raw.append('hasLink')
			if self.file:
				raw.append('hasFile')
			if self.poll:
				raw.append('hasPoll')
			if self.author or getText(self.author_field):
				raw.append('hasAuthor')
		raw += [self.forward_from, self.author, self.reply, 
			self.forward_author, getText(self.author_field), 
			self._getIndex()]
		return ' '.join([x for x in raw if x])

	def getAuthor(self):
		usernames = [self.forward_author, self.author]
		usernames = [item for item in usernames if isValidUsername(item)]
		if usernames:
			return usernames
		author = getText(self.author_field)
		if not author:
			return []
		return [author.replace(' ', '_')]

	def getKey(self):
		return '%s/%d' % (self.channel, self.post_id)

	def yieldPhotos(self):
		for item in self.soup.find_all('a', class_='tgme_widget_message_photo_wrap'):
			yield item['style'].split("background-image:url('")[1].split("')")[0]

	def __str__(self):
		return '%s: %s' % (self.getKey(), self.getMaintext())

def getPostFromSoup(name, soup):
	post = Post(name)
	post.soup = soup
	post.title = getField(soup, 'tgme_page_title', 
		'tgme_channel_info_header_title')
	post.description = getField(soup, 'tgme_page_description',
		'tgme_channel_info_description')
	post.reply = getAFieldSuffix(soup, 'tgme_widget_message_reply')
	reply_quote = soup.find('a', class_='tgme_widget_message_reply')
	if reply_quote:
		reply_quote.decompose()
	post.link = getField(soup, 'link_preview_title')
	post.file = getField(soup, 'tgme_widget_message_document_title')
	post.text = getField(soup, 'tgme_widget_message_text')
	post.poll = getField(soup, 'tgme_widget_message_poll')
	post.preview = getField(soup, 'link_preview_description')
	post.time = getTime(soup)
	post.forward_from = getForwardFrom(soup)
	post.author = getAFieldSuffix(soup, 'tgme_widget_message_author_name')
	post.author_field = soup.find('span', 'tgme_widget_message_author_name')
	post.forward_author = getAFieldSuffix(soup, 
		'tgme_widget_message_forwarded_from_name')
	post.is_group = isGroup(soup)
	return post
	